/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.data.DependencyManager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.data.DependencyManager"] = true;
/*
 * @author Ben
 */ 
dojo.provide("com.ibm.data.DependencyManager");

dojo.require("com.ibm.planner.SimplePOPlan");
/*
 * Provides data store item-dependency management tools for creating, managing, and
 * resolving dependencies between data store items.  For simplicity, this requires the
 * data store to implement the dojo.data.api.Identity API.
 * 
 * This manages *DIRECT* dependencies and should not be used to explicitly create 
 * indirect dependencies through chains.  Indirect, chained dependencies are managed 
 * internally and implicitly, therefore only direct dependencies should be explicitly 
 * added/removed.  Dependencies are unidirectional (one-way) relationships between
 * two items that visually look like one-way directed arrows in a graph between two
 * nodes where the dependent points to the superior.
 * 
 * DO NOT create cyclical dependencies as this will cause infinite recursion problems
 * which are not handled by this module.
 */

dojo.declare("com.ibm.data.DependencyManager",
	null,
	{
		constructor: function(store) {
			this._st = store;
			this._rels = {};
			// Map keyed by dependent ids with arrays whose indexes map to the dependency
			// type.  Each array item is another map of item ids that match the
			// superior in each dependency relationship.
			// {
			//		"sourceId": [
			//			{
			//				"targetId1": 1,
			//				"targetId2": 1
			//			},
			//			{
			//				"targetId1": 1
			//			}
			//		]
			// }

		},
		destroy: function() {
			delete this._st;
			delete this._rels;
		},
		_initItemDep: function(/*String*/ id) {
			var r = this._rels;
			if(!r[id]) {
				r[id] = [{}];
				for(var t in com.ibm.data.DependencyManager.types) {
					r[id].push({});
				}
			}
			return this;
		},
		updateDependencyId: function(/*String*/ oldId, /*String*/ newId) {
			// summary: Updates an id in the dependency map using its oldId
			//		and changing all dependencies in the map to use the newId.
			var r = this._rels;
			// update dependent relationships			
			if(r[oldId]) {
				r[newId] = r[oldId];
				delete r[oldId];
			}
			
			// update superior relationships
			for(var id in r) {
				for(var i = 0, typeR; typeR = r[id],i < r[id].length; i++) {
					if(typeR[oldId]) {
						typeR[newId] = typeR[oldId];
						delete typeR[oldId];
					}
				}
			}
			
			return this;
		},
		getDependencies: function(/*String*/ sourceId) {
			// summary: Returns an array of dependencies that this item has with other
			//		items.
			// sourceId: String id of the dependency source to search for dependencies.
			// returns: Array of dependencies whose indexes refer to the dependency
			//		type and whose objects at those indexes contain a map where the keys
			// 		are the dependency target ids.
			//		IMPORTANT -> DO NOT MODIFY THIS ARRAY!
			this._initItemDep(sourceId);
			return this._rels[sourceId];
		},
		getSourcesByType: function(/*String*/targetId, /*Number*/type) {
			var arr = this.getDependencies(targetId);
			var sources = [];
			if(arr) {
				for(var id in arr[0]) {
					if(arr[0][id][type]) sources.push(id);
				}
			}
			return sources;
		},
		getTargetsByType: function(/*String*/ sourceId, /*Number*/type) {
			// summary: Returns an array of all the targets of a source given a 
			// 		dependency type.
			// returns: Array of superior IDs in a particular dependency type
			type++;
			var arr = this.getDependencies(sourceId);
			var targets = [];
			if(arr) {
				for(var id in arr[type]) {
					targets.push(id);
				}
			}
			return targets;
		},
		addDependency: function(/*String*/ sourceId, /*String*/ targetId, /*Number*/ type) {
			var r = this._rels;
			type++;
			// source -> target
			this._initItemDep(sourceId);
			r[sourceId][type][targetId] = 1;
			
			// target -> source
			this._initItemDep(targetId);
			if(!r[targetId][0][sourceId]) r[targetId][0][sourceId] = [];
			r[targetId][0][sourceId][type - 1] = 1;
			return this;
		},
		removeDependency: function(/*String*/ sourceId, /*String*/ targetId, /*Number*/ type) {
			var r = this._rels;
			type++;
			if(r[sourceId] && r[sourceId][type][targetId]) {
				delete r[sourceId][type][targetId];
			}
			if(r[targetId] && r[targetId][0][sourceId] && r[targetId][0][sourceId][type]) {
				delete r[targetId][0][sourceId][type - 1];
			}
			return this;
		},
		getPlan: function(/*Array*/ items, /*Number*/ type) {
			// summary: Returns a partial-order plan which serializes the ordering of the items argument 
			// 		array for processing in sequences where dependencies are important.
			// 		The plan will also include any items that are not in the items array argument but
			// 		are in the superior chain of items in the arguments array.  Creating this plan can 
			// 		take some time if there are large dependency trees in the current type among the
			//		items in the arguments array.
			var plan = new com.ibm.planner.SimplePOPlan();
			dojo.forEach(items, function(dependentItem){
				this._buildPlan(plan, this._st.getIdentity(dependentItem), type);
			}, this);
			return plan;
		},
		_buildPlan: function(/*POPlan*/ plan, /*String*/ sourceId, /*Number*/ type) {
			if(!plan.hasStep(sourceId)) {
				var thisStep = plan.makeStep(sourceId, null);
				var supList = this.getTargetsByType(sourceId, type);
				for(var i=0; i < supList.length; i++) {
					var preStep = this._buildPlan(plan, supList[i], type);
					if(preStep) thisStep.addPreStep(preStep);
				}
				return thisStep;
			}
			else {
				return plan.getStep(sourceId);
			}
		}
	}
);

// separated "create" and "delete" for performance reasons
// adds a little bit of extra work for consumers, but its well-worth it
com.ibm.data.DependencyManager.types = {
	// create: Ensures that the dependency target is created before the source in any 
	// 		create actions.  Consumers may have to do special processing between 
	// 		create steps if a server-generated ID is used and needs to be mapped onto
	//		the dependent item before creating it on the server (for referential
	//		integrity).
	"create": 0,
	// delete: Ensures that if the dependency source is deleted, the target is too.  
	// 		Consumers may handle this differently depending on their implementation.  
	// 		Example 1: If an IO request is made to delete the source, it is assumed 
	// 		that the server also deletes the target, so the target is simply 
	// 		deleted on the client to synchronize with the server.
	//		Example 2: Each resource needs to be deleted separately, so a separate
	//		request is made to delete a resource once its source is deleted.
	"delete": 1,
	// load: Ensures that when the source loads, the target has loaded as well.
	//		For performance reasons, it is recommended to remove this dependency
	//		once the target has loaded (at any point) since it will no longer
	//		provide any value.  This can be used to keep things that are used 
	//		together loaded together.
	"load": 2,
	// order: Ensures that the target comes before the source in this type of
	// 		relationship when an ordered plan is built.  Keep in mind that if
	//		this creates an explicit dependency, so an ordered plan that potentially
	//		supports grouping will create a separate group for the target and source
	//		in these types of relationships when groups are formed because of this
	//		dependency.
	"order": 3
};

}
