/** Licensed Materials - Property of IBM, 5724-E76 and 5724-E77, (C) Copyright IBM Corp. 2009 - All Rights reserved.  **/

if(!dojo._hasResource["com.ibm.domUtilities"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["com.ibm.domUtilities"] = true;
/*
 * @author Ben
 */
dojo.provide( "com.ibm.domUtilities" );
/*
 * This module provides cross-browser functions for interacting with a DOM object to
 * create nodes and get and set attributes that may or may not be namespaced.  Since
 * browsers use various functions for interacting with namespaced DOM nodes and attributes,
 * this module abstracts those away and provides a single layer for using those operations.
 */

dojo.require( "dojox.data.dom" );
dojo.require( "com.ibm.portal.xslt" );

com.ibm.domUtilities = {
	
	constants: {
		// MSXML_NodeTypes: Constants for defining integer values for node type strings
		MSXML_NodeTypes: {
			element: 1,
			attribute: 2,
			text: 3,
			cdata: 4,
			entity_reference: 5,
			entity: 6,
			processing_instruction: 7,
			comment: 8,
			document: 9,
			document_type: 10,
			document_fragment: 11,
			notation: 12
		}
	},
	
	nsInfo: function(/*String*/ str, /*Object*/ namespaces) {
		// summary: Extracts the namespace URI, prefix, and local name of a QName according to this
		//		object's namespaces map that includes known prefixes and their namespace URIs.  If
		//		the string is not namespaced, then the namespaceURI and prefix are set to null, and
		//		the localName is set to the str as-is.
		// str: QName to extract namespace from
		// namespaces: Map of namespaces where keys are namespace prefixes and values are the namespace URIs.
		// returns: Map - {namespaceURI: namespaceURI, prefix: prefix, localName: localName}
		var parts = str.split(":");
		var prefix, localName;
		if(parts.length > 1) {
			prefix = parts[0];
			localName = parts[1];
			return { namespaceURI: namespaces[prefix], prefix: prefix, localName: localName};
		}
		else {
			return { namespaceURI: null, prefix: null, localName: str};
		}
	},
	getAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
		// summary: Gets an attribute of the given node, supporting namespaced and 
		// 		non-namespaced attributes.
		// node: Node to retrieve the attribute value from
		// attrName: The name of the attribute to get from the node.
		//		If the attribute is namespaced, it will be split into its parts using the : delimiter
		//		and the namespace used during lookup will be the one keyed in the namespace map.
		// namespaces: Map of prefixes to namespaceURIs
		// returns: Object value of attribute
		var ret = null;
		var parts = com.ibm.domUtilities.nsInfo(attrName, namespaces);
		if(parts.namespaceURI) {
			if(node.getAttributeNS) {
				ret = node.getAttributeNS( parts.namespaceURI, parts.localName );
			}
			else if(dojo.isIE) {
				ret = node.attributes.getQualifiedItem( parts.localName, parts.namespaceURI );
				if(ret) ret = ret.value;
			}
			else ret = node.getAttribute( attrName );
		}
		else {
			ret = node.getAttribute( attrName );
		}
		return ret;
	},
	setAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*String*/ value, /*Object*/ namespaces) {
		// summary: Sets an attribute of the given node, supporting namespaced and 
		// 		non-namespaced attributes.
		// node: Node to set the attribute value on
		// attrName: The name of the attribute to set on the node.
		//		If the attribute is namespaced, it will be split into its parts using the : delimiter
		//		and the namespace used during lookup will be the one keyed in the namespace map.
		// value: The actual value to store in the node at the key attribute specified by attrName.
		// namespaces: Map of prefixes to namespaceURIs
		if(value === true) value = "true";
		else if(value === false) value = "false";
		var parts = com.ibm.domUtilities.nsInfo(attrName, namespaces);
		if(parts.namespaceURI) {
			if(node.setAttributeNS) {
				node.setAttributeNS( parts.namespaceURI, attrName, value );
			}
			else if(dojo.isIE && node.ownerDocument) {
				var attrNode = node.ownerDocument.createNode(com.ibm.domUtilities.constants.MSXML_NodeTypes.attribute, attrName, parts.namespaceURI);
				attrNode.value = value;
				node.setAttributeNode(attrNode);
			}
			else node.setAttribute( attrName, value );
		}
        else node.setAttribute( attrName, value );
	},
	removeAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
		// summary: Removes an attribute of the given node, supporting namespaced and 
		// 		non-namespaced attributes.
		// node: Node to remove the attribute from
		// attrName: The name of the attribute to remove from the node.
		//		If the attribute is namespaced, it will be split into its parts using the : delimiter
		//		and the namespace used during lookup will be the one keyed in the namespaces map.
		// namespaces: Map of prefixes to namespaceURIs
		var parts = com.ibm.domUtilities.nsInfo(attrName, namespaces);
		if(parts.namespaceURI) {
			if(node.removeAttributeNS) {
				// for some reason we have to use the localName when getting the attribute even
				// though we have to use the full name when setting it
				node.removeAttributeNS( parts.namespaceURI, attrName );
			}
			else if(dojo.isIE) {
				node.attributes.removeQualifiedItem( parts.localName, parts.namespaceURI );
			}
			else node.removeAttribute( attrName );
		}
        else node.removeAttribute( attrName );
	},
	hasAttribute: function(/*DOMNode*/ node, /*String*/ attrName, /*Object*/ namespaces) {
		// summary: Checks if an attribute of the given node exists, supporting namespaced and 
		// 		non-namespaced attributes.
		// node: Node to check for the existence of the attribute from
		// attrName: The name of the attribute to check from the node.
		//		If the attribute is namespaced, it will be split into its parts using the : delimiter
		//		and the namespace used during lookup will be the one keyed in this item or store's 
		// 		namespace map.
		// namespaces: Map of prefixes to namespaceURIs
		// returns: Boolean true if attribute exists, false otherwise
		var ret = null;
		var parts = com.ibm.domUtilities.nsInfo(attrName, namespaces);
		if(parts.namespaceURI) {
			if(node.hasAttributeNS) {
				ret = node.hasAttributeNS( parts.namespaceURI, parts.localName );
			}
			else if(dojo.isIE) {
				ret = node.attributes.getQualifiedItem( parts.localName, parts.namespaceURI ) != null;
			}
			else ret = node.hasAttribute( attrName );
		}
		else if(!dojo.isIE) ret = node.hasAttribute( attrName );
		else ret = node.getAttributeNode( attrName ) != null;
		
		return ret;	
	},
	createElement: function(/*DOMDoc*/ doc, /*String*/ tagName, /*Object*/ namespaces, /*Object?*/ attrMap) {
		// summary: Cross-browser implementation for creating a namespaced node using the namespace
		//		map in this object to resolve prefixes to namespace URIs.
		//		If the tagName is not namespaced or a namespace URI cannot be resolved
		// 		using this object's namespaces map, a non-namespaced element will be created instead.
		// doc: DOMDoc document object to use to create the DOM node.
		// tagName: String name for the tag name of the element to create.  
		// 		Can be namespaced like prefix:localName or not like localName.
		// namespaces: Map of namespaces where keys are namespace prefixes and values are the namespace URIs.
		// attrMap: Optional argument specifying a key-value map of attributes to be
		// 		set on the node before returning it. 
		// returns: DOMNode A new DOM node that was created by doc
		var node = null;
		var parts = com.ibm.domUtilities.nsInfo(tagName, namespaces);
		if(parts.namespaceURI) {
			if(doc.createElementNS) {
				// W3C standard way of creating namespaced elements
				node = doc.createElementNS(parts.namespaceURI, tagName);
			}
			else if(dojo.isIE) {
				// we have to check dojo.isIE because IE throws an error when trying to check for
				// the existence of particular methods on the document object
				
				// this is IE's way of creating a namespaced node
				// use the tagName since it already includes the prefix as well
				node = doc.createNode("element", tagName, parts.namespaceURI);
			}
		}
		// if we don't have a node at this point, just create it without a namespace
		if(!node) node = doc.createElement(tagName);
		
		for(var name in attrMap) {
			com.ibm.domUtilities.setAttribute(node, name, attrMap[name], namespaces);
		}
		
		return node;
	},
	createFromJson: function(/*DOMDoc*/ doc, /*Object*/ def, /*Object*/ namespaces, /*DOMNode?*/ parentNode) {
		var obj = null;
		if(dojo.isString(def)){
			obj =  doc.createTextNode(def);
		}
		else {
			obj = com.ibm.domUtilities.createElement(doc, def.name, namespaces, def.attributes);
			dojo.forEach(def.children, function(childDef) {
				com.ibm.domUtilities.createFromJson(doc, childDef, namespaces, obj);
			});
		}
		
		if(parentNode) parentNode.appendChild(obj);
		return obj;
	},
	removeChildren: function(node) {
		while(node.hasChildNodes()) {
			var f = node.removeChild(node.firstChild);
			delete f;
		}
	},
	textContent: function(/*DOMNode*/ node, /*String?*/ text) {
		return dojox.data.dom.textContent.apply(null, arguments);
	},
	innerXML: function(/*DOMNode*/ node) {
		return dojox.data.dom.innerXML(node);
	},
	stringFromDoc: function(/*DOMNode*/ node) {
		return com.ibm.domUtilities.innerXML(node);
	},
	docFromString: function(/*String*/ str) {
		return com.ibm.portal.xslt.loadXmlString(str);
	},
	encodeXML: function(/*String*/ xmlStr, /*Boolean?*/ encodeQuotes) {
		// summary: Encodes a string for inclusion into an XML document so that any XML delimiters will
		//		be encoded as CDATA.
		if(dojo.isString(xmlStr)) {
			xmlStr = xmlStr.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;");
			if(encodeQuotes) xmlStr = xmlStr.replace(/'/gm, "&#039;").replace(/"/gm, "&#034;");
			return xmlStr;
		}
		else if(xmlStr != null) return xmlStr;
		else return "";
	},
	decodeXML: function(/*String*/ xmlStr, /*Boolean?*/ decodeQuotes) {
		if(dojo.isString(xmlStr)) {
			xmlStr = xmlStr.replace(/&lt;/gm, "<").replace(/&gt;/gm, ">").replace(/&amp;/gm, "&");
			if(decodeQuotes) xmlStr = xmlStr.replace(/&#039;/gm, "\'").replace(/&#034;/gm, "\"");
			return xmlStr;
		}
		else if(xmlStr != null) return xmlStr;
		else return "";
	}
}

}
