package fastforward;

import java.util.Locale;

/*
 * Copyright 2003 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Apr 9, 2003 3:55:48 PM
 */

/**
 * The AppContext represents the FastForward context defined by the application. The context is defined through the
 * J2EE application context as well as the application's WEB-INF/config.xml file.
 */

public interface AppContext extends ParameterSupport {
    /**
     * Returns the J2EE context path of the FastForward based application. This will always begin with a '/' and not end
     * with a '/'.
     * @return a String representing the context path of the current application
     */
    public String getContextPath();

    /**
     * Returns the native OS path to the given path relative to the application.
     * @param path a String specifying the relative path
     * @return a String representing the native real path
     */
    public String getRealPath(String path);

    /**
     * Returns the application's locale.
     * @return the locale in which the application is currently displayed
     */
    public Locale getLocale();

    /**
     * Returns an array of acceptable Locales for the associated user ordered by preference. Note that the
     * Locales are based on the request.getLocales() used to initialize the BloxContext.
     * @return an array of acceptable Locales for the associated user
     */
    public Locale[] getLocales();

    /**
     * Returns the application's base URL including the protocol, host, port, and context path.
     * @return a String representing the application's base URL
     */
    public String getAppURL();

    /**
     * Returns the relative path modified by the locale language and country information.
     * @param relativeURL The report object to reference
     * @return a String representing a URL to a localized version of the given URL
     */
    public String getLocalizedURL(String relativeURL);

    /**
     * Returns the path to the report page defined for the given Report.
     * @param report The report object to reference
     * @return a String representing a URL to display the given Report.
     */
    public String getReportURL(Report report);

    /**
     * Returns the path to the report's print page defined for the given Report. If no print-page is defined
     * for the report's template then the default print page will be used.
     * @param report The report object to reference
     * @return a String representing a URL to display the given Report's printable page.
     */
    public String getPrintURL(Report report);

    /**
     * Returns the path to the report's export to excel page defined for the given Report. If no excel-page is defined
     * for the report's template then the default excel page will be used.
     * @param report The report object to reference
     * @return a String representing a URL to display the given Report's export to excel page.
     */
    public String getExportToExcelURL(Report report);

    /**
     * Returns the path to the report's email page defined for the given Report. This page includes the report
     * header and footer text.
     * @param report The report object to reference
     * @return a String representing a URL to display the given Report's emailable page.
     */
    public String getMailURL(Report report);

    /**
     * Returns an object representing the currently logged in user.
     * @return a User object representing the currently logged in user
     */
    public User getUser();


    /**
     * Returns the SavedState object of the currently displayed report. This object represents the saved information for
     * user saved group and private reports. If there is no currently displayed report or if the displayed report is
     * public then this will return null;
     * @return a SavedState object for the currently displayed report if one exists
     */
    public SavedState getSavedState();

    /**
     * Returns the currently defined Blox render mode of the application as defined in the
     * WEB-INF/config.xml file of the application.
     * @return the Blox render mode of the application
     */
    public String getRenderMode();

    /**
     * Returns the currently defined theme of the application as defined in the WEB-INF/config.xml file.
     * @return the presentation theme of the application
     */
    public String getTheme();

    /**
     * Returns the currently defined title of the application as defined in the WEB-INF/config.xml file.
     * @return the application title
     */
    public String getTitle();

    /**
     * Returns the name of the logo image file as defined in the WEB-INF/config.xml file. This file is stored in the
     * application's images directory and is referenced as images/<logo file>.
     * @return the name of the logo file found in the application's images directory
     */
    public String getLogo();

    /**
     * Returns the footer text of the application as defined in the WEB-INF/config.xml file.
     * @return a String representing the HTML text to use as the application footer
     */
    public String getFooter();

    /**
     * Returns the J2EE user role used to define access to Admin tasks within the application. This role is defined in
     * the application's WEB-INF/config.xml file.
     * @return a String representing the name of the J2EE user role defining users with administrative access to the application
     */
    public String getAdminRole();

    /**
     * The email of the application administrator as defined in the WEB-INF/config.xml file. If the admin email is configured
     * then it is used to send feedback to the administrator.
     * @return a String representing the email address of the application administrator
     */
    public String getAdminEmail();

    /**
     * Returns the version of the FastForward this application is based on. The string represents the major and minor
     * version numbers. (i.e. "1.0")
     * @return a String representing the FastForward version of the application
     */
    public String getVersionString();

    /**
     * Add a message to the application log as found in the application's WEB-INF/fastforward.log file. The date and
     * current user will be prepended to the message text.
     * @param type  message type. The following constants can be used:
     * <ul>
     *      <li>LogFile.INTERNAL</li>
     *      <li>LogFile.DEBUG</li>
     *      <li>LogFile.VERBOSE</li>
     *      <li>LogFile.INFO</li>
     *      <li>LogFile.SYSTEM</li>
     *      <li>LogFile.WARNING</li>
     *      <li>LogFile.ERROR</li>
     *      <li>LogFile.FATAL</li>
     *      <li>LogFile.ASSERTION</li>
     * </ul>
     * @param message the text to append to the application's log file.
     */
    public void log(int type, String message);

    /**
     * Add a message to the application log file which includes a Throwable object. The log entry will include the stack trace
     * of the throwable object and any nested causes.
     * @param type  message type. The following constants can be used:
     * <ul>
     *      <li>LogFile.INTERNAL</li>
     *      <li>LogFile.DEBUG</li>
     *      <li>LogFile.VERBOSE</li>
     *      <li>LogFile.INFO</li>
     *      <li>LogFile.SYSTEM</li>
     *      <li>LogFile.WARNING</li>
     *      <li>LogFile.ERROR</li>
     *      <li>LogFile.FATAL</li>
     *      <li>LogFile.ASSERTION</li>
     * </ul>
     * @param message the text to append to the application's log file.
     * @param throwable the Throwable object
     */
    public void log(int type, String message, Throwable throwable);

     /**
      * Find a localized string in the FastForwardBundle.properties file.
      * @param key
      * @return null if the String is not found
      */
    public String getString(String key);

    /**
      * Find a localized string in the FastForwardBundle.properties file.
      * @param key
      * @param args object array for replacement
      * @return null if the String is not found
      */
    public String getString(String key, Object args[]);
}
