/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-L14
 *  Copyright IBM Corp. 2005
 * The source code for this program is not published or otherwise divested
 *   of its trade secrets, irrespective of what has been deposited with the
 *   U.S. Copyright Office.
 *
 * User: dmessink
 * Date: Sep 9, 2002 12:35:06 PM
 */

package fastforward;

import javax.servlet.ServletException;

import java.io.*;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

public class LogFile {
    // Message types
    static public final int INTERNAL = 0;           // Internal message (not available to end users)
    static public final int DEBUG = 1;              // Debug - messages to aid debugging the system
    static public final int VERBOSE = 2;			// Verbose - more detailed information
    static public final int INFO = 3;               // Information - messages supplying verbose information
    static public final int SYSTEM = 4;             // System - messages for major system events (new sessions, etc.)
    static public final int WARNING = 5;            // Warning - message to indicate that a mistake was made or
                                                    //   a resource is becoming scarce
    static public final int ERROR = 6;              // Error - message to indicate that an operation can not be completed

    static public final int FATAL = 7;              // Fatal - message indicates a non-recoverable error - the system will
                                                    //  be terminated
    static public final int ASSERTION = 8;          // Programming errors

    static public final int FIRST = INTERNAL;
    static public final int LAST = ASSERTION;

    private String name;
    private File file;
    private Object fileLock=new Object();
    private int logLevel = INFO;    // FastForward log level, default to INFO

    private static final String LINEFEED="\r\n";
    private DateFormat stampFormatter;

    public LogFile(String name) {
         this(name,Locale.getDefault());
    }

    public LogFile(String name,Locale locale) {
        this.name=name;
        stampFormatter = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.LONG,locale);
    }

    public int getLogLevel() {
        return logLevel;
    }

    public void setLogLevel(int logLevel) {
        this.logLevel = logLevel;
    }

    public File getFile() {
        return file;
    }

    public void setFile(File file) {
        this.file = file;
    }

    public void writeln(int type, String message) throws IOException {
        if (type < logLevel) return;

        synchronized (fileLock) {
            Writer writer=null;

            try {
                writer = getWriter();

                writeMessage(writer, message);
                writer.flush();
            } finally {
                if (writer != null)
                    writer.close();
            }
        }
    }

    public void writeln(int type, String message, Throwable exc) throws IOException {
        if (type < logLevel) return;

        synchronized (fileLock) {
            Writer writer=null;

            try {
                writer = getWriter();

                writeMessage(writer, message);

                Throwable exception=exc;

                if (exception != null) {
                    String excMessage = exception.getMessage();

                    if (excMessage == null)
                        excMessage = exception.getClass().getName();

                    writer.write(excMessage);
                    writer.write(LINEFEED);

                    exception.printStackTrace(new PrintWriter(writer));

                    if (exception instanceof ServletException)
                        exception = ((ServletException)exception).getRootCause();
                    else
                        exception = exception.getCause();

                    if (exception != null) {
                        writer.write("Root Cause:");
                        writer.write(LINEFEED);

                        excMessage = exception.getMessage();
                        if (excMessage == null)
                            excMessage = exception.getClass().getName();

                        writer.write(excMessage);
                        writer.write(LINEFEED);

                        exception.printStackTrace(new PrintWriter(writer));
                    }
                }

                writer.flush();
            } finally {
                if (writer != null)
                    writer.close();
            }
        }
    }

    public void clear() throws IOException {
        if (file.exists())
            file.delete();
    }

    private void writeMessage(Writer writer, String message) throws IOException {
        writer.write(getTimestamp());

        if (name != null) {
            writer.write(" (");
            writer.write(name);
            writer.write(")");
        }

        writer.write(": ");
        writer.write(message);
        writer.write(LINEFEED);
    }

    private Writer getWriter() throws IOException {
        if (file == null)
            throw new RuntimeException("LogFile file property is not set");

        FileOutputStream fos = new FileOutputStream(file.getPath(), true);
        OutputStreamWriter writer = new OutputStreamWriter(fos, "UTF-8");

        return writer;
    }

    private String getTimestamp() {
        return stampFormatter.format(new Date());
    }
}
