/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Sep 13, 2002 1:41:32 AM
 */
package fastforward;

import fastforward.Acl;

import java.util.*;

public class NavigationTree {
    public final static String ROOT_ID = "publicNavigation"; // keep this around for admin
    public final static String PRIVATE_ROOT_ID = "privateNavigation";
    public final static String GROUP_ROOT_ID = "groupNavigation";

    private Folder root;

    public Folder getNavigationRoot() {
        return root;
    }

    public void setNavigationRoot(Folder root) {
        this.root = root;
    }

    public Folder getNavigationRoot(User user) {
        if (user == null)
            throw new IllegalArgumentException("User cannot be null");
        return filterNavigation(null, user, root);
    }


    private Folder filterNavigation(Folder parent, User user, Report report) {
        if (report.isFolder()) {
            Folder container = (Folder) report;
            Acl acl = container.getAcl();

            if (acl == null || acl.hasAccess(user)) {
                Folder newFolder = new Folder();
                container.copyTo(newFolder);

                Iterator childrenList = container.getChildren().iterator();

                while (childrenList.hasNext()) {
                    filterNavigation(newFolder, user, (Report) childrenList.next());
                }

                // Don't add an empty container or container with no reports to the tree
                if (container.getChildren().isEmpty() || newFolder.collectItems().size() == 0)
                    return parent;

                if (parent != null)
                    parent.add(newFolder);

                return newFolder;
            }
        } else {
            Report clonedItem=new Report();
            report.copyTo(clonedItem);
            parent.add(clonedItem);
        }

        return parent;
    }

    public Report find(String id) {
        Report report=findFolder(id);

        return (report != null) ? report : findItem(id);
    }

    public Folder findFolder(String id) {
        return root.findFolder(id);
    }

    public Report findItem(String id) {
        return root.findItem(id);
    }

    private String getUniqueItemId(String reportIdPrefix) {
        int idKey = 1;
        String id = (reportIdPrefix == null) ? ("Report" + idKey) : (reportIdPrefix + "_Report" + idKey);

        while (root.findItem(id) != null) {
            idKey++;
            id = (reportIdPrefix == null) ? ("Report" + idKey) : (reportIdPrefix + "_Report" + idKey);
        }

        return id;
    }

    private String getUniqueFolderId() {
        int idKey = 1;
        String id = "Folder" + idKey;

        while (findFolder(id) != null) {
            idKey++;
            id = "Folder" + idKey;
        }

        return id;
    }

    public Folder createNewFolder(Folder parent) {
        String id = getUniqueFolderId();

        Folder container = new Folder(id, id, null);

        if (parent != null)
            parent.add(container);
        else
            root.add(container);

        return container;
    }

    public Report createNewItem(Folder parent) {
        return createNewItem(parent, null);
    }

    public Report createNewItem(Folder parent, String reportIdPrefix) {
        String id = getUniqueItemId(reportIdPrefix);

        Report report = new Report(id, id, null);

        if (parent != null)
            parent.add(report);
        else
            root.add(report);

        return report;
    }

    public void move(Report report, Folder toFolder, int position) {
        Folder fromFolder = report.getParent();

        fromFolder.delete(report);

        toFolder.insertReport(report, position);
    }

    public void prettyPrint() {
        root.prettyPrint(0);
    }

    public static class Report {
        private String id;
        private String label;
        private String description;
        private Folder parent;

        public Report() {
        }

        public Report(String id, String label, String description) {
            this.id = id;
            this.label = label;
            this.description = description;
        }

        public void setId(String id) {
            this.id = id;
        }

        public String getId() {
            return id;
        }

        public void setParent(Folder parent) {
            this.parent = parent;
        }

        public Folder getParent() {
            return parent;
        }

        public void setLabel(String label) {
            this.label = label;
        }

        public String getLabel() {
            return label;
        }

        public void setDescription(String description) {
            this.description = description;
        }

        public String getDescription() {
            return description;
        }

        public void copyTo(Report newFolder) {
            newFolder.setId(getId());
            newFolder.setLabel(getLabel());
            newFolder.setDescription(getDescription());
        }

        public boolean isRoot() {
            return (parent == null);
        }

        public boolean isFolder() {
            return false;
        }

        public void delete() {
            parent.delete(this);
        }

        public void prettyPrint(int indent) {
            for (int x=0; x<indent; x++) {
                System.out.print("  ");
            }

            System.out.println(getId()+" "+getLabel()+" "+this);
        }
    }

    public static class Folder extends Report {
        private Vector children = new Vector();
        private Acl acl;

        public Folder() {
        }

        public Folder(String id, String label, String description) {
            super(id, label, description);
        }

        public void add(Folder folder) {
            children.addElement(folder);
            folder.setParent(this);
        }

        public void add(Report report) {
            children.addElement(report);
            report.setParent(this);
        }

        public void delete(Report report) {
            children.remove(report);
        }

        public Acl getAcl() {
            return acl;
        }

        public void setAcl(Acl acl) {
            this.acl = acl;
        }

        public List getChildren() {
            return Collections.unmodifiableList(children);
        }

        public Report getChild(int i) {
            if (children.size() < i + 1)
                return null;

            return (Report) children.get(i);
        }

        public boolean isFolder() {
            return true;
        }

        public Folder findFolder(String id) {
            if (id.equals(getId()))
                return this;

            Iterator childList = children.iterator();
            while (childList.hasNext()) {
                Report report = (Report) childList.next();

                if (!report.isFolder())
                    continue;

                Folder foundFolder = ((Folder) report).findFolder(id);
                if (foundFolder != null)
                    return foundFolder;
            }

            return null;
        }

        public Report findItem(String id) {
            Iterator childList = children.iterator();
            while (childList.hasNext()) {
                Report report = (Report) childList.next();

                if (!report.isFolder()) {
                    if (id.equals(report.getId()))
                        return report;
                } else {
                    Report foundItem = ((Folder) report).findItem(id);
                    if (foundItem != null)
                        return foundItem;
                }
            }

            return null;
        }

        public Vector collectItems() {
            Vector items = new Vector();
            Iterator childList = children.iterator();
            while (childList.hasNext()) {
                Report report = (Report) childList.next();

                if (!report.isFolder())
                    items.addElement(report);
                else {
                    items.addAll(((Folder) report).collectItems());
                }
            }
            return items;
        }

        public Vector collectFolders() {
            Vector items = new Vector();
            Iterator childList = children.iterator();
            while (childList.hasNext()) {
                Report report = (Report) childList.next();

                if (report.isFolder()) {
                    items.addElement(report);
                    items.addAll(((Folder) report).collectFolders());
                }
            }
            return items;
        }

        public void prettyPrint(int indent) {
            super.prettyPrint(indent);
            Iterator childList=children.iterator();
            while (childList.hasNext()) {
                ((Report)childList.next()).prettyPrint(indent+1);
            }
        }

        public void insertReport(Report report, int position) {
            if (position >= 0)
                children.insertElementAt(report,position);
            else
                children.addElement(report);

            report.setParent(this);
        }
    }
}
