package fastforward;

import com.alphablox.blox.Blox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.form.IFormBlox;

import java.util.Vector;

/*
 * Copyright 2003 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Feb 4, 2003 3:45:03 PM
 */

/**
 * Represents the report data as entered in the report definition page of the application admin page. Provides access
 * to the report name, description, template, and report parameters for use in validating the report.
 */

public class ReportData {
    private Template template;
    private String reportName;
    private String description;
    private IFormBlox[] formBlox;
    private Vector errors;
    private AppContext appContext;

    protected ReportData(Template template, String reportName, String description, IFormBlox[] formBlox, AppContext appContext) {
        this.template = template;
        this.reportName = reportName;
        this.description = description;
        this.formBlox = formBlox;
        this.appContext = appContext;
    }

    /**
     * Returns the Template object that the report data is based on.
     * @return the Template object selected in the report definition page
     */
    public Template getTemplate() {
        return template;
    }

    /**
     * Returns the report name entered by the administrator into the report definition page.
     * @return a String representing the display name of the report
     */
    public String getReportName() {
        return reportName;
    }

    /**
     * Returns the report description entered by the administrator into the report definition page.
     * @return a String representing the description of the report
     */
    public String getDescription() {
        return description;
    }

    /**
     * Returns an array of the FormBlox as defined in the report's edit page. These formblox represent the report
     * parameter values entered by the administrator.
     * @return an array of FormBlox objects
     */
    public IFormBlox[] getFormBlox() {
        return formBlox;
    }

    /**
     * Returns a FormBlox object by name. The name of the FormBlox matches the named report parameter as defined
     * in the report template.
     * @param parameterName the name of the FormBlox to access
     * @return the FormBlox used to enter the named report parameter value
     * @throws ServerBloxException
     */
    public IFormBlox getFormBlox(String parameterName) throws ServerBloxException {
        for (int x = 0; x < formBlox.length; x++) {
            IFormBlox blox = formBlox[x];
            if (blox instanceof Blox)
                if (parameterName.equals(blox.getBloxName()))
                    return blox;
        }

        return null;
    }

    /**
     * Returns the values currently entered for the named report parameter.
     * @param parameterName the named parameter in which to obtain the current values
     * @return an array of Strings representing the current report parameter values
     * @throws ServerBloxException
     */
    public String[] getParameterValues(String parameterName) throws ServerBloxException {
        IFormBlox blox = getFormBlox(parameterName);

        if (blox == null)
            return null;

        return blox.getFormValues();
    }

    /**
     * Returns the value currently entered for the named report parameter. If multiple values are defined only the first
     * one is returned.
     * @param parameterName the named parameter in which to obtain the current value
     * @return a String representing the current report parameter value
     * @throws ServerBloxException
     */
    public String getParameterValue(String parameterName) throws ServerBloxException {
        IFormBlox blox = getFormBlox(parameterName);

        if (blox == null)
            return null;

        return blox.getFormValue();
    }

    /**
     * Adds the given error text to the error dialog to be presented to the administrator. If no errors are reported
     * then the report data is considered to be valid.
     * @param errorText the error text to display to the administrator explaining the error
     */
    public void addError(String errorText) {
        if (errors == null)
            errors = new Vector();

        errors.addElement(errorText);
    }

    /**
     * Returns any and all errors reported by the ReportValidator. If no errors are reported during validation
     * then the report data is considered to be valid.
     * @return an array of Strings representing the errors discovered during report data validation
     */
    public String[] getErrors() {
        if (errors == null)
            return null;

        return (String[]) errors.toArray(new String[errors.size()]);
    }

    /**
     * Returns the fastforward AppContext.
     */
    public AppContext getAppContext() {
        return appContext;
    }
}
