/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Nov 12, 2002 8:01:35 PM
 */
package fastforward;

import fastforward.Parameter;

/**
 * Defines the parameters, presentation, and edit page used by reports generated using this report template. Templates
 * are located in the application's template directory and are defined by the template's template.xml file.
 */
public interface Template {
    /**
     * Returns the name of the Template. The name of the template is the name of the template folder under the
     * application's template directory.
     * @return a String representing the name of the template
     */
    public String getName();

    /**
     * Returns the display name of the Template. The display name is used to display the template to the administrator
     * in the report definition admin page.
     * @return a String representing the template's human readable display name
     */
    public String getDisplayName();

    /**
     * Returns the description of the Template. The description is used to describe the template to the administrator
     * in the report definition admin page.
     * @return a String representing the template's description
     */
    public String getDescription();

    /**
     * Returns the Parameter object defining the named Template parameter. Template parameters are defined in the
     * template.xml file.
     * @param name the name of the Parameter object to access
     * @return the Parameter object defining the named template parameter
     */
    public Parameter getParameter(String name);

    /**
     * Returns an array of Parameter objects as defined by the Template. Template parameters are defined in the
     * template.xml file.
     * @return an array of Parameter objects defined in this Template
     */
    public Parameter[] getParameters();

    /**
     * Returns the URL to the template's report page used to display a report. This URL is relative to the application
     * and is defined by the report-page tag in the template.xml file.
     * @return a String representing the relative URL to the report's display page
     */
    public String getReportURL();

    /**
     * Returns the URL to the template's print page used to preview a report. This URL is relative to the application
     * and is defined by the print-page tag in the template.xml file. If no print-page is defined the
     * report page will be used.
     * @return a String representing the relative URL to the report's print preview page
     */
    public String getPrintPageURL();

    /**
     * Returns the URL to the template's excel page used to export a report. This URL is relative to the application
     * and is defined by the excel-page tag in the template.xml file. If no excel-page is defined the
     * report page will be used.
     * @return a String representing the relative URL to the report's export to excel page
     */
    public String getExcelPageURL();

    /**
     * Returns the URL to the template's help page used to export a report. This URL is relative to the application
     * and is defined by the help-page tag in the template.xml file.
     * @return a String representing the relative URL to the report's help page
     */
    public String getHelpPageURL();

    /**
     * Assigns a ReportValidator object to this template. The ReportValidator is used to validate the report parameters
     * entered by the application administrator when defining reports based on this template.
     * @param validator a ReportValidator object used to validate the report data entered in the report definition page
     */
    public void setValidator(ReportValidator validator);
}
