/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 */
package fastforward;

import com.alphablox.util.XmlFile;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.transform.TransformerException;
import java.io.*;
import java.util.Enumeration;
import java.util.Locale;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class TemplateImpl implements Template {
    private String name;
    private String displayName;
    private String description;
    private String reportPage;
    private String editPage;
    private String printPage;
    private String excelPage;
    private Parameter[] parameters;
    private ReportValidator validator;
    private String helpPage;
    private Locale locale;
    private String[] localeTargets;

    public TemplateImpl() {
        this(Locale.getDefault());
    }

    public TemplateImpl(Locale locale) {
        this.locale = locale;
    }

    public String getName() {
        return name;
    }

    public String getDisplayName() {
        return displayName;
    }

    public String getDescription() {
        return description;
    }

    public String getReportURL() {
        return getURL(reportPage);
    }

    public String getPrintPageURL() {
        return getURL(printPage);
    }

    public String getExcelPageURL() {
        return getURL(excelPage);
    }

    public String getEditPageURL() {
        return getURL(editPage);
    }

    public String getHelpPageURL() {
        return getURL(helpPage);
    }

    private String getURL(String page) {
        if (page == null)
            return null;

        return AppContextImpl.TEMPLATE_DIRECTORY + "/" + getName() + "/" + page;
    }

    public Parameter[] getParameters() {
        return parameters;
    }

    public Parameter getParameter(String parameterName) {
        for (int x = 0; x < parameters.length; x++) {
            if (parameterName.equals(parameters[x].getName()))
                return parameters[x];
        }
        return null;
    }

    private void loadTemplate(File templateDir, String templateName) throws TemplateException {
        try {
            Document document = getDocument(templateDir, templateName);

            this.name = templateName;

            // display-name and description are localizable -- first look for current localization
            // then values without an xml:lang (localization) attribute

            displayName = getLocalizedValue(document, "/template/display-name");
            if (displayName == null)
                displayName = name;

            description = getLocalizedValue(document, "/template/description");

            Node node = XPathAPI.selectSingleNode(document, "/template/report-page/node()");
            reportPage = node.getNodeValue();

            node = XPathAPI.selectSingleNode(document, "/template/edit-page/node()");
            editPage = (node != null) ? node.getNodeValue() : null;

            node = XPathAPI.selectSingleNode(document, "/template/print-page/node()");
            printPage = (node != null) ? node.getNodeValue() : null;

            node = XPathAPI.selectSingleNode(document, "/template/excel-page/node()");
            excelPage = (node != null) ? node.getNodeValue() : null;

            node = XPathAPI.selectSingleNode(document, "/template/help-page/node()");
            helpPage = (node != null) ? node.getNodeValue() : null;

            Node reportParams = XPathAPI.selectSingleNode(document, "/template/report-params");
            parseReportParams(reportParams);
        } catch (Exception exc) {
            throw new TemplateException("Error loading template " + templateName, exc);
        }
    }

    private String getLocalizedValue(Node reference, String path) throws TransformerException {
        String[] targets = getLocaleTargets();
        Node node = null;

        for (int x = 0; x < targets.length; x++) {
            String target = targets[x];

            node = XPathAPI.selectSingleNode(reference, path + "[@xml:lang='" + target + "']/node()");
            if (node != null)
                break;
        }

        if (node == null)
            node = XPathAPI.selectSingleNode(reference, path + "[not(@xml:lang)]/node()");

        if (node == null)
            node = XPathAPI.selectSingleNode(reference, path + "[@xml:lang='en']/node()");

        return (node != null) ? node.getNodeValue() : null;
    }

    private String[] getLocaleTargets() {
        if (localeTargets != null)
            return localeTargets;

        localeTargets = AppContextImpl.getLocaleTargets(locale);

        return localeTargets;
    }

    private void parseReportParams(Node reportParams) throws TransformerException {
        if (reportParams == null) {
            parameters = new Parameter[0];
            return;
        }

        NodeList paramList = XPathAPI.selectNodeList(reportParams, "param");
        parameters = new Parameter[paramList.getLength()];
        for (int x = 0; x < paramList.getLength(); x++) {
            parameters[x] = parseParameter(paramList.item(x));
        }
    }

    private Parameter parseParameter(Node param) throws TransformerException {
        Node node = XPathAPI.selectSingleNode(param, "param-name/node()");
        String name = (node != null) ? node.getNodeValue() : null;

        String label = getLocalizedValue(param, "param-label");
        if (label == null)
            label = name;

        NodeList nodes = XPathAPI.selectNodeList(param, "default-value/node()");
        String[] defaultValues = new String[nodes.getLength()];
        for (int x = 0; x < nodes.getLength(); x++) {
            defaultValues[x] = nodes.item(x).getNodeValue();
        }

        return new ParameterImpl(name, label, defaultValues);
    }

    private Document getDocument(File templatesDir, String templateName) throws XmlParseException {
        File xmlPath = new File(templatesDir, templateName + "/template.xml");

        XmlFile xmlFile = new XmlFile();
        xmlFile.setXmlFile(xmlPath);
        xmlFile.setDTDFile(new File(templatesDir, "/template.dtd"));
        XMLErrorHandler errorHandler = new XMLErrorHandler(xmlFile);
        xmlFile.setErrorHandler(errorHandler);

        Document document = xmlFile.getDocument();

        if (errorHandler.exceptionThrown())
            throw errorHandler.getException();

        return document;
    }

    public static TemplateImpl getTemplate(File templateDir, String templateName, Locale locale) throws TemplateException {
        expandZippedTemplates(templateDir);

        TemplateImpl template = new TemplateImpl(locale);

        File directory = new File(templateDir, templateName);

        if (!directory.exists())
            throw new TemplateException("Template " + templateName + " not found");

        template.loadTemplate(templateDir, templateName);

        return template;
    }

    /**
     * Returns an array of the available templates. We always read the templates from disk.
     * @param templateDir
     * @return
     */
    public static Template[] getTemplates(File templateDir, Locale locale) throws TemplateException {
        expandZippedTemplates(templateDir);

        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if (!file.isDirectory())
                    return false;

                File xmlFile = new File(file, "template.xml");

                return xmlFile.exists();
            }
        };

        File[] templateDirectories = templateDir.listFiles(filter);
        Template[] templates = new Template[templateDirectories.length];

        for (int x = 0; x < templateDirectories.length; x++) {
            File templateDirectory = templateDirectories[x];
            TemplateImpl template = new TemplateImpl(locale);

            template.loadTemplate(templateDir, templateDirectory.getName());

            templates[x] = template;
        }

        return templates;
    }

    private static void expandZippedTemplates(File templateDir) throws TemplateException {

        try {
            FileFilter zipFilter = new FileFilter() {
                public boolean accept(File file) {
                    return file.getName().endsWith(".zip");
                }
            };

            File[] zipFiles = templateDir.listFiles(zipFilter);

            for (int x = 0; x < zipFiles.length; x++) {
                File zipFile = zipFiles[x];
                String templateName = zipFile.getName().substring(0, zipFile.getName().length() - 4);
                File dir = new File(templateDir, templateName);

                if (!dir.exists()) {
                    expandZipTemplate(new ZipFile(zipFile), dir);
                }
            }
        } catch (IOException exc) {
            exc.printStackTrace();
            throw new TemplateException(exc);
        }
    }

    private static void expandZipTemplate(ZipFile zipFile, File directory) throws IOException {
        Enumeration entries = zipFile.entries();

        directory.mkdir();

        while (entries.hasMoreElements()) {
            ZipEntry zipEntry = (ZipEntry) entries.nextElement();
            File entryFile = new File(directory, zipEntry.getName());

            if (zipEntry.isDirectory()) {
                continue;
            }

            entryFile.getParentFile().mkdirs();

            FileOutputStream outStream = new FileOutputStream(entryFile);
            InputStream inStream = zipFile.getInputStream(zipEntry);
            byte[] buffer = new byte[4 * 1024];
            int len;

            while ((len = inStream.read(buffer, 0, buffer.length)) > 0) {
                outStream.write(buffer, 0, len);
            }

            outStream.close();
            inStream.close();
        }

        zipFile.close();
    }

    public void setValidator(ReportValidator validator) {
        this.validator = validator;
    }

    public ReportValidator getValidator() {
        return validator;
    }

    public void copyTo(TemplateImpl existingTemplate) {
        existingTemplate.name = name;
        existingTemplate.displayName = displayName;
        existingTemplate.description = description;
        existingTemplate.reportPage = reportPage;
        existingTemplate.editPage = editPage;
        existingTemplate.printPage = printPage;
        existingTemplate.excelPage = excelPage;
        existingTemplate.parameters = parameters;
    }

    public static class ParameterImpl implements Parameter {
        private String name;
        private String label;
        private String[] defaultValues;

        public ParameterImpl(String name, String label, String[] defaultValues) {
            this.name = name;
            this.label = label;
            this.defaultValues = defaultValues;
        }

        public String getName() {
            return name;
        }

        public String getLabel() {
            return label;
        }

        public String getDefaultValue() {
            if (defaultValues == null ||
                    defaultValues.length == 0)
                return null;

            return defaultValues[0];
        }

        public String[] getDefaultValues() {
            return defaultValues;
        }
    }

}
