/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Dec 10, 2002 3:58:24 PM
 */
package fastforward;

import com.alphablox.blox.ContainerBlox;
import com.alphablox.blox.ServerBloxException;
import com.alphablox.blox.form.FormEvent;
import com.alphablox.blox.form.FormEventListener;
import com.alphablox.blox.form.TreeFormBlox;
import com.alphablox.blox.uimodel.BloxModel;
import com.alphablox.blox.uimodel.core.*;
import com.alphablox.blox.uimodel.core.event.ClickEvent;
import com.alphablox.blox.uimodel.core.event.IEventHandler;
import com.alphablox.util.Message;

import java.net.URLEncoder;
import java.io.UnsupportedEncodingException;

public class UIControl {
    private AppContextImpl appContext;
    private TreeFormBlox treeBlox;
    private String visibility = Report.PUBLIC;
    // private RadioButtonFormBlox navigationSwitcher;
    private FastForwardTabs navigationSwitcher;
    private Toolbar toolbar;
    private TreeFormBlox.Folder privateRoot;
    private TreeFormBlox.Folder groupRoot;
    private TreeFormBlox.Folder publicRoot;
    private Static reportTitle;
    private Static reportDescription;
    private Toolbar helpToolbar;
    private Static reportHelpLink;

    public UIControl() {
    }

    public void init(AppContextImpl appContext) {
        this.appContext = appContext;
        buildTree(Report.PUBLIC);
        buildTree(Report.PRIVATE);
        buildTree(Report.GROUP);
    }

    private void buildTree(IConfiguration config) {
        if (config instanceof UserConfiguration)
            buildTree(Report.PRIVATE);
        else if (config instanceof AppConfiguration)
            buildTree(Report.PUBLIC);
        else
            buildTree(Report.GROUP);
    }

    public void buildTree(String treeType) {
        TreeFormBloxHelper helper = new TreeFormBloxHelper();
        if (treeType.equals(Report.PRIVATE)) {
            privateRoot = helper.createTree(appContext.getUserConfiguration(), appContext.getUser(),appContext.getResourceBundle());
            privateRoot.setExpanded(true);
        } else if (treeType.equals(Report.PUBLIC)) {
            publicRoot = helper.createTree(appContext.getAppConfiguration(), appContext.getUser(),appContext.getResourceBundle());
            publicRoot.setExpanded(true);
        } else {
            groupRoot = helper.createGroupTree(appContext.getGroupConfigurations(), appContext.getUser(), appContext.getResourceBundle());
            groupRoot.setExpanded(true);
        }
    }

    public void setTreeBlox(TreeFormBlox treeBlox) {
        this.treeBlox = treeBlox;
        if (treeBlox != null) {
            treeBlox.addFormEventListener(new NavigationTreeListner());
        }
        updateTree();
    }

    public String getVisibility() {
        return visibility;
    }

    private void setNavigationRoot(NavigationTree navigationTree) {
        if (navigationTree.getNavigationRoot() == null) {
            NavigationTree.Folder root = new NavigationTree.Folder("privateNavigation", appContext.getString("PRIVATE_REPORT_LABEL"), appContext.getString("PRIVATE_REPORT_DESC"));
            navigationTree.setNavigationRoot(root);
        }
    }

    public void moveItem(String itemID, String folderID, int position, IConfiguration targetConfig) throws Exception {
        targetConfig.readConfig();

        NavigationTree navigationTree = targetConfig.getNavigationTree();
        setNavigationRoot(navigationTree);
        NavigationTree.Report report = navigationTree.find(itemID);
        NavigationTree.Folder parent = navigationTree.findFolder(folderID);

        navigationTree.move(report, parent, position);

        saveAndUpdate(targetConfig);
    }

    public void createReport(ReportImpl srcReport, String reportLabel, String reportDesc, NavigationTree.Folder parent,
                             SavedStateImpl handler, IConfiguration targetConfig) throws Exception {

        // reload the config file before update.
        // may be we should lock the config file until we finish saving it?
        targetConfig.readConfig();

        NavigationTree navigationTree = targetConfig.getNavigationTree();
        setNavigationRoot(navigationTree);
        String groupName = null;
        if (targetConfig instanceof GroupConfiguration)
            groupName = ((GroupConfiguration) targetConfig).getGroup();
        NavigationTree.Report reportNode = navigationTree.createNewItem(parent, groupName);
        ReportImpl newReport = new ReportImpl();
        newReport.setId(reportNode.getId());
        newReport.setOwner(appContext.getUser().getName());
        newReport.setLabel(reportLabel);
        newReport.setDescription(reportDesc);
        newReport.setTemplate((TemplateImpl) srcReport.getTemplate());
        newReport.setConfig(targetConfig);
        String[] params = srcReport.getParameterNames();
        for (int i = 0; i < params.length; i++) {
            newReport.setParameter(params[i], srcReport.getParameterValues(params[i]));
        }
        SavedStateData newSavedState = new SavedStateData();
        newReport.setSavedStateData(newSavedState);

        reportNode.setLabel(reportLabel);
        reportNode.setDescription(reportDesc);

        // save the blox and controls
        if (groupName == null)
            handler.save(newReport.getId(), newReport.getSavedStateData(), Report.PRIVATE);
        else
            handler.save(newReport.getId(), newReport.getSavedStateData(), Report.GROUP);

        targetConfig.addReport(newReport);

        saveAndUpdate(targetConfig);
    }

    private void saveAndUpdate(IConfiguration targetConfig) throws Exception {
        saveConfigFile(targetConfig);
        updateTree(targetConfig);
    }

    public void updateTree(IConfiguration targetConfig) {
        buildTree(targetConfig);
        updateTree();
    }

    private void updateTree() {
        if (treeBlox == null)
            return;

        try {
            if (visibility.equals(Report.PRIVATE))
                treeBlox.setRoot(privateRoot);
            else if (visibility.equals(Report.PUBLIC))
                treeBlox.setRoot(publicRoot);
            else
                treeBlox.setRoot(groupRoot);

            treeBlox.setRootVisible(true);
            ReportImpl currentReport = (ReportImpl) appContext.getCurrentReport();
            if (currentReport != null && visibility.equals(getReportVisibility(currentReport))) {
                TreeFormBlox.Item item = treeBlox.find(currentReport.getId());
                treeBlox.setSelected(item);
            } else {
                treeBlox.setSelected((TreeFormBlox.Item) null);
            }

            treeBlox.changed();
        } catch (ModelException e) {
            appContext.log(LogFile.ERROR, "Unexpected model exception ", e);
        }
    }

    public void clearReport() {
        try {
            appContext.setCurrentReport(null);
            updateTree();
            updateToolbar();
            updateToolbarHelp();
            updateTitle();
            updateDescription();
            updateHelpLink();
        } catch (ServerBloxException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    private void saveConfigFile(IConfiguration targetConfig) throws Exception {
        targetConfig.saveConfig();
    }

    public void deleteReport(ReportImpl report) throws Exception {
        if (report == null) return;
        try {
            IConfiguration targetConfig = report.getConfig();
            // reload the config file before update.
            // may be we should lock the config file until we finish saving it?
            targetConfig.readConfig();

            // security check
            String loggedInUser = appContext.getUser().getName();
            if (!loggedInUser.equals(report.getOwner()))
                throw new SecurityException(appContext.getString("ERROR_ONLY_OWNER_DELETES") );
            targetConfig.deleteReport(report.getId());

            saveAndUpdate(targetConfig);
            appContext.setCurrentReport(null);
        } catch (Exception exc) {
            appContext.log(LogFile.ERROR, appContext.getString("ERROR_DELETING_REPORT"), exc);
            throw exc;
        }
    }

    public void showDialog(Dialog dialog) throws ServerBloxException, ModelException {
        IModelDispatcher dispatcher = treeBlox.getBloxModel().getDispatcher();
        dispatcher.showDialog(dialog);
    }


    class ConfirmDeleteDialog extends Dialog {
        private ReportImpl report;

        public ConfirmDeleteDialog(ReportImpl report, String title, String prompt) {
            super();
            this.report = report;
            try {
                loadFromResource(appContext.getResourcePath("confirm.xml"), appContext.getLocales() );
                setTitle(title);
                Static text = (Static) searchForComponent("prompt");
                text.setTitle(prompt);
                Button okBtn = (Button) searchForComponent("ok");
                okBtn.setSelected(true);
                setController(new DialogController(this) {
                    public void actionOk() {
                        deleteItem(dialog.getDispatcher());
                        super.actionOk();
                    }
                });
            } catch (Exception e) {
                appContext.log(LogFile.ERROR, "Unexpected model exception", e);
            }
        }

        void deleteItem(IModelDispatcher dispatcher) {
            try {
                deleteReport(report);
                appContext.setCurrentReport(null);
                dispatcher.sendClientCommand(appContext.getCallbackFunction("clearReport()"));
                dispatcher.sendClientCommand(appContext.getCallbackFunction("pollUI()"));
            } catch (Exception e) {
                appContext.log(LogFile.ERROR, appContext.getString("ERROR_DELETING_REPORT_DETAIL",new Object[]{report.getId()}), e);
                MessageBox.message(this, "Delete Error", e.getMessage());
            }
        }
    }

    public TreeFormBlox getTreeBlox() {
        return treeBlox;
    }

    public void setContext(AppContextImpl appContext) {
        this.appContext = appContext;
    }

    public void setNavigationSwitcher(FastForwardTabs navigationSwitcher) {
        this.navigationSwitcher = navigationSwitcher;
    }

    public boolean handleNewTab(String selectedTab) {
        try {
            // Bug 16159 : To fix this bug, we don't clear the report upon switching the tab.
            // Any currently selected report will be there as it is.
//            if (appContext.getCurrentReport() != null) {
//                appContext.setCurrentReport(null);
//                // call javascript to clear the report being displayed in the UI
//                this.navigationSwitcher.getBloxModel().getDispatcher().sendClientCommand(appContext.getCallbackFunction("clearReport()"));
//            }

            setVisibility(selectedTab);
            updateTree();
            updateToolbar();
            updateToolbarHelp();
            updateTitle();
            updateDescription();
            updateHelpLink();
            this.navigationSwitcher.getBloxModel().getDispatcher().sendClientCommand(appContext.getCallbackFunction("pollUI()"));
        } catch (ServerBloxException sbe) {
            sbe.printStackTrace();
        } catch (ModelException me) {
            me.printStackTrace();
        }
        return true;
    }

    public void setVisibility(String visibility) {
        this.visibility = visibility;
//        if (visibility.equalsIgnoreCase("public"))
//            this.visibility = Report.PUBLIC;
//        else if (visibility.equalsIgnoreCase("private"))
//            this.visibility = Report.PRIVATE;
//        else
//            this.visibility = Report.GROUP;
    }

    public void setToolbarBlox(ContainerBlox blox) throws Exception {
        toolbar = (Toolbar) ModelResource.loadResource(appContext.getResourcePath("toolbar.xml"), appContext.getLocales() );
        toolbar.setRolloverEnabled(true);

        BloxModel model = blox.getBloxModel();
        model.clear();
        toolbar.setName("ffwdToolbar");
        toolbar.setThemeClass("clsFFWDToolbar");

        Controller controller = toolbar.getController();
        if (controller == null) {
            controller = new Controller() {
            };
            toolbar.setController(controller);
        }
        ToolbarEventHandler myHandler = new ToolbarEventHandler(model.getDispatcher(), appContext);
        controller.addEventHandlerToEnd(myHandler);

        model.add(toolbar);
        updateToolbar();
    }

    public void setToolbarHelpBlox(ContainerBlox blox) throws Exception {
        helpToolbar = (Toolbar) ModelResource.loadResource(appContext.getResourcePath("toolbarhelp.xml"), appContext.getLocales() );
        helpToolbar.setRolloverEnabled(true);

        BloxModel model = blox.getBloxModel();
        model.clear();
        helpToolbar.setName("ffwdToolbarHelp");
        helpToolbar.setThemeClass("clsFFWDToolbar");


        Controller controller = helpToolbar.getController();
        if (controller == null) {
            controller = new Controller() {
            };
            helpToolbar.setController(controller);
        }
        ToolbarEventHandler myHandler = new ToolbarEventHandler(model.getDispatcher(), appContext);
        controller.addEventHandlerToEnd(myHandler);

        model.add(helpToolbar);
    }

    public void updateToolbar() {
        ReportImpl report = (ReportImpl) appContext.getCurrentReport();
        boolean onOff = (report == null || report.getTemplate() instanceof AppContextImpl.MissingTemplate);
        String reportVisibility = getReportVisibility(report);

        try {
            if (toolbar != null) {
                ToolbarButton save = (ToolbarButton) toolbar.findInContainer("save");
                if (save != null) {
                    save.setDisabled(onOff);
                    save.changed();
                }

                ToolbarButton delete = (ToolbarButton) toolbar.findInContainer("delete");
                if (delete != null) {
                    if (report == null || reportVisibility.equals(Report.PUBLIC) || (report != null && !report.getOwner().equals(appContext.getUser().getName())))
                        delete.setDisabled(true);
                    else
                        delete.setDisabled(onOff);
                    delete.changed();
                }

                ToolbarButton email = (ToolbarButton) toolbar.findInContainer("email");
                if (email != null) {
                    if (reportVisibility.equals(Report.PRIVATE))
                        email.setDisabled(true);
                    else
                        email.setDisabled(onOff);
                    email.changed();
                }
                ToolbarButton print = (ToolbarButton) toolbar.findInContainer("print");
                if (print != null) {
                    print.setDisabled(onOff);
                    print.changed();
                }
                ToolbarButton excel = (ToolbarButton) toolbar.findInContainer("excel");
                if (excel != null) {
                    excel.setDisabled(onOff);
                    excel.changed();
                }
                toolbar.changed();
                ToolbarButton admin = (ToolbarButton) toolbar.findInContainer("admin");
                if (admin != null) {
                    if (appContext.getUser().isAdministrator() == false)
                        admin.delete();
                    else {
                        admin.setDisabled(false);
                        admin.changed();
                    }

                }
                toolbar.changed();
            }
        } catch (ModelException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    public void updateToolbarHelp() {
        try {
            ToolbarButton help = (ToolbarButton) toolbar.findInContainer("help");
            if (help != null)
                help.changed();
            helpToolbar.changed();
        } catch (ModelException e) {
            e.printStackTrace();  //To change body of catch statement use Options | File Templates.
        }
    }

    private String getReportVisibility(ReportImpl report) {
        if (report == null)
            return visibility;  // if there is no current report, return the visibility based on current tab

        IConfiguration config = report.getConfig();
        if (config instanceof AppConfiguration)
            return Report.PUBLIC;
        else if (config instanceof UserConfiguration)
            return Report.PRIVATE;
        else
            return Report.GROUP;
    }

    private String encode(String string) {
        try {
            return URLEncoder.encode(string, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return string;
        }
    }

    public void setReportTitleBlox(ContainerBlox blox) throws Exception {
        reportTitle = new Static();

        BloxModel model = blox.getBloxModel();
        reportTitle.setName("ffwdReportTitle");
        reportTitle.setThemeClass("clsFFWDReportTitle");

        model.add(reportTitle);
        updateTitle();
    }

    public void updateTitle() {
        ReportImpl report = (ReportImpl) appContext.getCurrentReport();
        String title = report == null ? "" : report.getLabel();

        if (reportTitle != null) {
            reportTitle.setTitle(title);
        }
        try {
            reportTitle.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"updateTitle"}));

        }
    }

    public void updateTitleLoading() {
        ReportImpl report = (ReportImpl) appContext.getCurrentReport();
        String title = report == null ? "" : appContext.getString("LOADING_REPORT",new Object[]{report.getLabel()}) ;

        if (reportTitle != null) {
            reportTitle.setTitle(title);
        }
        try {
            reportTitle.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"updateTitleLoading"}));

        }
    }

    public void updateDescription() {
        ReportImpl report = (ReportImpl) appContext.getCurrentReport();
        String title = report == null ? "" : report.getDescription();

        if (reportDescription != null) {
            reportDescription.setTitle(title);
        }
        try {
            reportDescription.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"updateDescription"}));
        }
    }

    public void clearDescription() {

        if (reportDescription != null) {
            reportDescription.setTitle("");
        }
        try {
            reportDescription.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"clearDescription"}));
        }
    }

    public void clearHelpLink() {

        if (reportHelpLink != null) {
            reportHelpLink.setTitle("");
            reportHelpLink.setClientLink(null);
        }
        try {
            reportHelpLink.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"clearHelpLink"}));
        }
    }

    public void setReportDescriptionBlox(ContainerBlox blox) throws Exception {
        reportDescription = new Static();

        BloxModel model = blox.getBloxModel();
        reportDescription.setName("ffwdReportDescription");
        reportDescription.setThemeClass("clsFFWDReportDescription");
        model.add(reportDescription);
        updateDescription();
    }

    public void setHelpLinkBlox(ContainerBlox blox) throws Exception {
        reportHelpLink = new Static("");

        BloxModel model = blox.getBloxModel();
        reportHelpLink.setThemeClass("clsFFWDHelpLink");
        model.add(reportHelpLink);
        updateHelpLink();
    }

    public void updateHelpLink() {
        String url = null;
        ReportImpl report = (ReportImpl) appContext.getCurrentReport();
        if (report != null) {
            TemplateImpl template = (TemplateImpl) report.getTemplate();
            url = template == null ? null : template.getHelpPageURL();
        }
        if (reportHelpLink != null) {
            if (url == null) {
                reportHelpLink.setTitle("");
                reportHelpLink.setClientLink(null);
            } else {
                reportHelpLink.setTitle(appContext.getString("REPORT_HELP"));
                reportHelpLink.setClientLink(new ClientLink(appContext.getLocalizedURL(url), "help"));
            }
        }
        try {
            reportHelpLink.changed();
        } catch (ModelException e) {
            new Message(Message.ERROR, "UIControl", e);
            throw new IllegalStateException(appContext.getString("ERROR_IN_METHOD",new Object[]{"updateHelpLink"}));
        }
    }

    class NavigationTreeListner implements FormEventListener {
        public void valueChanged(FormEvent event) throws Exception {
            if (event.getFormBlox() == treeBlox) {
                String selectedReport = treeBlox.getFormValue();
                TreeFormBlox.Item item = treeBlox.find(selectedReport);
                if (item instanceof TreeFormBlox.Folder) return; // don't do anything
                if (item != null) {
                    ReportImpl report = (ReportImpl) item.getUserObject();
                    if (!report.equals(appContext.getCurrentReport())) {
                        appContext.getAdminControl().clearPreviewReport();
                        String url = appContext.getReportURL(report);
                        treeBlox.getBloxModel().getDispatcher().sendClientCommand(appContext.getCallbackFunction("showReport('" + url + "')"));
                    }
                }
            }
        }
    }

    class ToolbarEventHandler implements IEventHandler {
        IModelDispatcher dispatcher;
        AppContextImpl context;

        public ToolbarEventHandler(IModelDispatcher dispatcher, AppContextImpl context) {
            this.dispatcher = dispatcher;
            this.context = context;
        }

        public boolean handleClickEvent(ClickEvent event) throws ModelException {
            String button = event.getComponent().getName();
            ReportImpl report = (ReportImpl) appContext.getCurrentReport();
            if (button.equals("email")) {
                String url = appContext.getMailURL(report);

                String subject = appContext.getAppConfiguration().getParameter("MailtoSubject", report.getLabel());
                String mailURI = "mailto:?subject=" + encode(subject).replace('+', ' ') + "&body=" + encode(url);
                dispatcher.showBrowserWindow(new ClientLink(mailURI));
            } else if (button.equals("excel")) {
                String url = appContext.getExportToExcelURL(report);
                dispatcher.showBrowserWindow(new ClientLink(url, "ExcelWindow"));
            } else if (button.equals("print")) {
                String url = appContext.getPrintURL(report);
                dispatcher.showBrowserWindow(new ClientLink(url, "PrintWindow"));
            } else if (button.equals("feedback")) {
                String adminEmail = appContext.getAppConfiguration().getParameter("AdminEmail");
                String url;
                if (adminEmail != null && (!"".equals(adminEmail))) {
                    url = "mailto:" + adminEmail;
                    dispatcher.showBrowserWindow(new ClientLink(url, "PrintWindow"));
                } else {
                    MessageBox.message(toolbar, appContext.getString("NO_EMAIL_TITLE"), appContext.getString("APP_ERROR_NO_EMAIL"));
                }
            } else if (button.equals("help")) {
                String url = appContext.getLocalizedURL("help/help.html");
                dispatcher.showBrowserWindow(new ClientLink(url, "HelpWindow"));
            } else if (button.equals("save")) {
                SaveAsDialog dialog = new SaveAsDialog();
                dialog.init(context);
                dispatcher.showDialog(dialog);
            } else if (button.equals("delete")) {
                ConfirmDeleteDialog confirm = new ConfirmDeleteDialog(report, appContext.getString("DELETE_REPORT_CONFIRM"), appContext.getString("DELETE_REPORT_YN"));
                dispatcher.showDialog(confirm);
            } else if (button.equals("admin")) {
                try {
                    if (appContext.getAdminControl().okToDisplayAdminPage()) {
                        dispatcher.sendClientCommand(appContext.getCallbackFunction("displayAdminPage()"));
                    }
                } catch (Exception e) {
                    new Message(Message.FATAL, "UIControl", e).send();
                    throw new IllegalStateException(appContext.getString("ERROR_DISPLAYING_ADMIN"));
                }
            }
            return false;
        }
    }
}
