/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: rraghavan
 * Date: Sep 4, 2002
 * Time: 9:30:15 AM
 */
package fastforward;

import com.alphablox.blox.*;
import com.alphablox.server.Constants;

import java.util.Map;
import java.io.File;

import fastforward.ParameterSupport;
import org.w3c.dom.Document;

public class UserImpl implements ParameterSupport, User {
    public static final String USER_XML_FILE = "userconfig";
    public static final String GROUP_XML_FILE = "groupconfig";

    private String name; // name will be null for Guest users
    private IRepositoryBlox repoBlox;
    private boolean isAdministrator=false;

    public void init(IRepositoryBlox repBlox) {
        this.repoBlox = repBlox;
        try {
            name = repoBlox.getUserProperty("Name");
        } catch (ServerBloxException e) {
            e.printStackTrace();
        }
    }

    public String getName() {
        return name;
    }

    public void deleteConfigFile(String contextName) throws ServerBloxException {
        if (name != null) {
            if (repoBlox.exists(RepositoryBlox.VISIBILITY_PRIVATE, name, contextName + File.separator + USER_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT)) {
                repoBlox.delete(RepositoryBlox.VISIBILITY_PRIVATE, name, contextName + File.separator + USER_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT);
            }
        }
    }

    public void deleteConfigFile(String groupName, String contextName) throws ServerBloxException {
        if (groupName != null) {
            if (repoBlox.exists(RepositoryBlox.VISIBILITY_GROUP, groupName, contextName + File.separator + GROUP_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT)) {
                repoBlox.delete(RepositoryBlox.VISIBILITY_GROUP, groupName, contextName + File.separator + GROUP_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT);
            }
        }
    }

    public Document readConfigFile(String contextName) throws ServerBloxException {
        if (name != null) {
            if (repoBlox.exists(RepositoryBlox.VISIBILITY_PRIVATE, name, contextName + File.separator + USER_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT)) {
                Document document = (Document) repoBlox.load(RepositoryBlox.VISIBILITY_PRIVATE, name, contextName + File.separator + USER_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT);
                return document;
            }
        }
        return null;
    }

    public Document readConfigFile(String groupName, String contextName) throws ServerBloxException {
        if (groupName != null) {
            if (repoBlox.exists(RepositoryBlox.VISIBILITY_GROUP, groupName, contextName + File.separator + GROUP_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT)) {
                Document document = (Document) repoBlox.load(RepositoryBlox.VISIBILITY_GROUP, groupName, contextName + File.separator + GROUP_XML_FILE, RepositoryBlox.TYPE_XMLDOCUMENT);
                return document;
            }
        }
        return null;
    }

    public void saveConfigFile(Document xmlDocument, String contextName) throws ServerBloxException{
        if (name != null)
            repoBlox.save(RepositoryBlox.VISIBILITY_PRIVATE, name, contextName + File.separator + USER_XML_FILE, xmlDocument, RepositoryBlox.TYPE_XMLDOCUMENT);
    }

    public void saveConfigFile(String groupName, Document xmlDocument, String contextName) throws ServerBloxException{
        if (groupName != null)
            repoBlox.save(RepositoryBlox.VISIBILITY_GROUP, groupName, contextName + File.separator + GROUP_XML_FILE, xmlDocument, RepositoryBlox.TYPE_XMLDOCUMENT);
    }

    protected void setIsAdministrator(boolean isAdministator) {
        this.isAdministrator=isAdministator;
    }

    public void setParameter(String paramName, String paramValue) {
        try {
            repoBlox.setUserProperty(paramName, paramValue);
        } catch (ServerBloxException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    public void setParameter(String paramName, String[] paramValues) {
        throw new IllegalArgumentException("Not Supported");
    }

    public String getParameter(String paramName) {
        try {
            return repoBlox.getUserProperty(paramName);
        } catch (ServerBloxException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    public String getParameter(String paramName, String defaultValue) {

        try {
            String value = repoBlox.getUserProperty(paramName);
            if (value == null)
                return defaultValue;

            return value;
        } catch (ServerBloxException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    public String[] getParameterValues(String parameterName) {
        return new String[]{getParameter(parameterName)};
    }

    public String[] getParameterNames() {
        try {
            Map params = repoBlox.getUserPropertyMap();
            String[] paramNames = new String[params.size()];

            params.keySet().toArray(paramNames);

            return paramNames;
        } catch (ServerBloxException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    public String[] getGroups() {
        try {
            AdminBlox adminBlox = new AdminBlox();
            com.alphablox.blox.repository.User user;
            if(name != null) {
                user = adminBlox.getUser(name);
            }
            else {
                user = adminBlox.getUser(Constants.GUEST_USERNAME);
            }
            String[] groups = user.getGroupNames();

            return groups;
        }
        catch (ServerBloxException e) {
            throw new RuntimeException(e.getMessage());
        }
    }

    public boolean isInGroup(String groupName) {
        String[] groups;

        try {
            AdminBlox adminBlox = new AdminBlox();
            com.alphablox.blox.repository.User user;
            if(name != null) {
                user = adminBlox.getUser(name);
            }
            else {
                user = adminBlox.getUser(Constants.GUEST_USERNAME);
            }
            groups = user.getGroupNames();
        }
        catch (Exception exc) {
            return false;
        }

        for (int i = 0; i < groups.length; i++) {
            if (groups[i].equalsIgnoreCase(groupName))
                return true;
        }

        return false;
    }

    public boolean isAdministrator() {
        return isAdministrator;
    }
}


