/*
 * Copyright 2002 Alphablox Corporation. All rights reserved.
 * User: dmessink
 * Date: Sep 15, 2002 9:31:31 PM
 */
package fastforward;

import com.alphablox.util.HTMLUtils;
import org.apache.xpath.XPathAPI;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.TransformerException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Vector;

public class XMLConfiguration {
    private IConfiguration config;

    private static final String XML_FFWDVERSION = "version";
    private static final String XML_ID = "id";
    private static final String XML_LABEL = "label";
    private static final String XML_DESCRIPTION = "description";
    private static final String XML_ACL = "acl";
    private FastForwardResourceBundle resource;

    public XMLConfiguration() {
    }

//    public XMLConfiguration(Resource resource) {
//        this.resource = resource;
//    }

    public void init(IConfiguration config, FastForwardResourceBundle resource) {
        this.config = config;
        this.resource = resource;
    }

    public void updateConfig(Document xmlDocument) throws XmlParseException {
        parseDocument(xmlDocument);
    }

    public Document generateXmlDocument() throws XmlParseException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.newDocument();

            Element docElement = document.createElement("fastforward");
            docElement.setAttribute(XML_FFWDVERSION, config.getAppContext().getVersionString());
            document.appendChild(docElement);

            appendAppParameters(document, docElement);
            appendReports(document, docElement);
            appendNavigation(document, docElement);
            return document;
        } catch (Exception exc) {
            throw new XmlParseException(exc);
        }
    }

    private void parseDocument(Document xmlDocument) throws XmlParseException {
        checkVersion(xmlDocument);
        parseParameters(xmlDocument);
        parseReports(xmlDocument);
        parseNavigation(xmlDocument);
        validateConfiguration(xmlDocument);
        setReportData(config.getNavigationTree().getNavigationRoot());
    }

    private void checkVersion(Document xmlDocument) throws XmlParseException {
        Node root;
        try {
            root = XPathAPI.selectSingleNode(xmlDocument, "/fastforward");
        } catch (Exception exc) {
            throw new XmlParseException(exc);
        }
        if (root != null) {
            Node namedItem = root.getAttributes().getNamedItem(XML_FFWDVERSION);
            if (namedItem == null)
                throw new IllegalArgumentException(resource.getString("ERROR_NO_VERSION"));
            String version = namedItem.getNodeValue();
            if (!version.equals(config.getAppContext().getVersionString())) {
                throw new IllegalArgumentException(resource.getString("ERROR_BAD_VERSION", new Object[]{version, config.getAppContext().getVersionString()}));
            }
        }
    }

    private void parseParameters(Document xmlDocument) throws XmlParseException {
        try {
            Node params = XPathAPI.selectSingleNode(xmlDocument, "/fastforward/app-params");

            if (params == null)
                return;

            NodeList paramList = params.getChildNodes();
            if (paramList != null && paramList.getLength() > 0)
                config.getParameterSet().parseParameters(paramList);
        } catch (Exception exc) {
            throw new XmlParseException(exc);
        }
    }

    private void parseReports(Document xmlDocument) throws XmlParseException {
        NodeList reportList = null;

        try {
            Node reports = XPathAPI.selectSingleNode(xmlDocument, "/fastforward/reports");
            if (reports == null)
                throw new XmlParseException(resource.getString("ERROR_NO_REPORTS_TAG"));

            reportList = reports.getChildNodes();

            config.getReports().clear();

            for (int index = 0; index < reportList.getLength(); index++) {
                Node reportElement = reportList.item(index);
                String name = XPathAPI.selectSingleNode(reportElement, "@id").getNodeValue();

                String templateName = XPathAPI.selectSingleNode(reportElement, "template/node()").getNodeValue();
                // default the owner to be admin. in the previous version of config.xml there is no owner element
                // and to be compatible with that, let's make those reports owned by admin user.
                String owner = "admin";
                Node ownerNode = XPathAPI.selectSingleNode(reportElement, "owner/node()");
                if (ownerNode != null)
                    owner = ownerNode.getNodeValue();

                ReportImpl report = new ReportImpl();
                report.setId(name);
                report.setOwner(owner);

                try {
                    TemplateImpl template = config.getTemplate(templateName);
                    report.setTemplate(template);
                } catch (TemplateException exc) {
                    throw new XmlParseException("Error loading template " + templateName, exc);
                }

                ParameterSet reportParams = new ParameterSet();
                reportParams.parseParameters(XPathAPI.selectNodeList(reportElement, "param"));

                Node ssNode = XPathAPI.selectSingleNode(reportElement, "saved-state");
                if (ssNode != null) {
                    SavedStateData savedState = new SavedStateData();
                    savedState.parse(ssNode);
                    report.setSavedStateData(savedState);
                }

                report.setParameters(reportParams);

                config.getReports().addElement(report);
            }
        } catch (TransformerException e) {
            throw new XmlParseException(e);
        }
    }

    private void parseNavigation(Document xmlDocument) throws XmlParseException {
        try {
            Node navigationElement = XPathAPI.selectSingleNode(xmlDocument, "/fastforward/navigation");
            String rootId = navigationElement.getAttributes().getNamedItem(XML_ID).getNodeValue();
            String rootNodeLabel = navigationElement.getAttributes().getNamedItem(XML_LABEL).getNodeValue();

            NavigationTree.Folder root = new NavigationTree.Folder(rootId, null, null);
            root.setAcl(new Acl(null));
            config.getNavigationTree().setNavigationRoot(root);
            root.setLabel(rootNodeLabel);

            NodeList nodeList = navigationElement.getChildNodes();

            for (int index = 0; index < nodeList.getLength(); index++) {
                parseNavigationElement(nodeList.item(index), root);
            }
        } catch (TransformerException e) {
            throw new XmlParseException(e);
        }
    }

    private boolean isItem(Node node) {
        Element element;
        element = (Element) node;
        return "item".equals(element.getTagName());
    }

    private void parseNavigationElement(Node child, NavigationTree.Folder container) throws XmlParseException {
        Element element = (Element) child;

        if (isItem(child))
            parseItem(element, container);
        else
            parseContainer(element, container);
    }

    private void parseContainer(Element child, NavigationTree.Folder parent) throws XmlParseException {
        String id = child.getAttribute(XML_ID);
        String name = child.getAttribute(XML_LABEL);
        String description = child.getAttribute(XML_DESCRIPTION);

        NavigationTree.Folder container = new NavigationTree.Folder(id, name, description);
        parent.add(container);

        Acl acl = parseAcl(child, parent.getAcl());

        container.setAcl(acl);

        NodeList nodeList = child.getChildNodes();

        for (int index = 0; index < nodeList.getLength(); index++) {
            if (!XML_ACL.equals(nodeList.item(index).getNodeName()))
                parseNavigationElement(nodeList.item(index), container);
        }
    }

    private Acl parseAcl(Element containerElement, Acl parentAcl) throws XmlParseException {
        Acl acl = new Acl(parentAcl);

        try {
            Node aclElement = XPathAPI.selectSingleNode(containerElement, "acl");

            // The acl element is not required
            if (aclElement == null)
                return acl;

            NodeList groupList = XPathAPI.selectNodeList(aclElement, "group/node()");

            String[] groups = new String[groupList.getLength()];
            for (int i = 0; i < groupList.getLength(); i++) {
                groups[i] = groupList.item(i).getNodeValue();
            }

            acl.setGroups(groups);
        } catch (TransformerException e) {
            throw new XmlParseException(e);
        }

        return acl;
    }

    private void parseItem(Element child, NavigationTree.Folder container) {
        String id = child.getAttribute(XML_ID);
        String name = child.getAttribute(XML_LABEL);
        String description = child.getAttribute(XML_DESCRIPTION);

        container.add(new NavigationTree.Report(id, name, description));
    }

    private void setReportData(NavigationTree.Folder container) {
        Iterator childList = container.collectItems().iterator();

        while (childList.hasNext()) {
            NavigationTree.Report item = (NavigationTree.Report) childList.next();

            ReportImpl report = getReport(item.getId());
            Acl acl = item.getParent().getAcl();

            // ensures that the top level items also have an acl
            if (acl == null)
                acl = new Acl(null);

            report.setAcl(acl);
            report.setLabel(item.getLabel());
            report.setDescription(item.getDescription());
            report.setConfig(config);
        }
    }

    private ReportImpl getReport(String reportID) {
        Iterator reportList = config.getReports().iterator();
        while (reportList.hasNext()) {
            ReportImpl report = (ReportImpl) reportList.next();
            if (reportID.equals(report.getId()))
                return report;
        }
        return null;
    }


    private void validateConfiguration(Document xmlDocument) throws XmlParseException {
        validateReportsIntegrity(xmlDocument);
    }

    private void validateReportsIntegrity(Document xmlDocument) throws XmlParseException {
        Vector reportList = (Vector) config.getReports().clone();

        for (int x = 0; x < reportList.size(); x++) {
            ReportImpl report = (ReportImpl) reportList.elementAt(x);

            if (report.getTemplate() == null)
                throw new XmlParseException(resource.getString("ERROR_NO_TEMPLATE", new Object[]{report.getId()}));
        }

        try {
            NodeList itemList = XPathAPI.selectNodeList(xmlDocument, "/fastforward/navigation//item");

            for (int index = 0; index < itemList.getLength(); index++) {
                Node item = itemList.item(index);
                String reportID = item.getAttributes().getNamedItem(XML_ID).getNodeValue();

                ReportImpl report = getReport(reportID);
                if (report == null) {
                    throw new XmlParseException(resource.getString("ERROR_REPORT_NOT_FOUND", new Object[]{reportID}));
                }

                reportList.remove(report);
            }
        } catch (TransformerException e) {
            throw new XmlParseException(e);
        }

        if (!reportList.isEmpty()) {
            ReportImpl report = (ReportImpl) reportList.get(0);

            throw new XmlParseException(resource.getString("ERROR_REPORT_NOT_IN_NAV", new Object[]{report.getId()}));
        }
    }

    private void appendAppParameters(Document document, Element parent) {
        if (config.getParameterSet().getParameterNames().length == 0)
            return;
        Element params = document.createElement("app-params");
        parent.appendChild(params);

        appendParamTags(document, params, config.getParameterSet());
    }

    private void appendReports(Document document, Element parent) {
        Element reportsTag = document.createElement("reports");
        parent.appendChild(reportsTag);

        Iterator reportList = config.getReports().iterator();
        while (reportList.hasNext()) {
            ReportImpl report = (ReportImpl) reportList.next();

            Element reportTag = document.createElement("report");
            reportTag.setAttribute("id", report.getId());
            reportsTag.appendChild(reportTag);

            Element template = createTextElement(document, "template", report.getTemplate().getName());
            reportTag.appendChild(template);

            Element owner = createTextElement(document, "owner", report.getOwner());
            reportTag.appendChild(owner);

            appendParamTags(document, reportTag, report);
            appendSavedState(document, reportTag, report);
        }
    }

    private void appendSavedState(Document document, Element reportTag, ReportImpl report) {
        SavedStateData state = report.getSavedStateData();
        if (state == null)
            return;
        Element savedStateNode = document.createElement("saved-state");
        Element controlsNode = document.createElement("controls");
        SavedStateData.Control[] controls = state.getControls();

        for (int i = 0; i < controls.length; i++) {
            Element controlNode = document.createElement("control");
            SavedStateData.Control control = controls[i];
            Element controlname = document.createElement("control-name");
            Text text = document.createTextNode(control.getName());
            controlname.appendChild(text);

            Element controlvalue = document.createElement("control-value");
            text = document.createTextNode(control.getValue());
            controlvalue.appendChild(text);
            controlNode.appendChild(controlname);
            controlNode.appendChild(controlvalue);
            controlsNode.appendChild(controlNode);
        }
        savedStateNode.appendChild(controlsNode);

        SavedStateData.Bookmark[] marks = state.getBookmarks();
        Element bookmarks = document.createElement("bookmarks");
        for (int i = 0; i < marks.length; i++) {
            Element blox = document.createElement("blox");
            SavedStateData.Bookmark mark = marks[i];
            Element bloxname = document.createElement("blox-name");
            Text text = document.createTextNode(mark.getBloxName());
            bloxname.appendChild(text);

            Element bookmarkname = document.createElement("bookmark");
            text = document.createTextNode(mark.getBookmarkName());
            bookmarkname.appendChild(text);

            blox.appendChild(bloxname);
            blox.appendChild(bookmarkname);
            bookmarks.appendChild(blox);

            if (mark.isReportBlox()) {
                Element isReportBlox = document.createElement("isReportBlox");
                text = document.createTextNode("true");
                isReportBlox.appendChild(text);
                blox.appendChild(isReportBlox);
            }
        }

        savedStateNode.appendChild(bookmarks);

        reportTag.appendChild(savedStateNode);
    }

    /**
     * create navigation root from scratch!
     * @param document
     * @param parent
     */
    private void appendNavigation(Document document, Element parent) {
        Element navigation = document.createElement("navigation");
        NavigationTree.Folder root = config.getNavigationTree().getNavigationRoot();
        navigation.setAttribute(XML_ID, root.getId());
        navigation.setAttribute(XML_LABEL, root.getLabel());
        parent.appendChild(navigation);

        Iterator childList = root.getChildren().iterator();
        while (childList.hasNext()) {
            appendNavigation(document, navigation, (NavigationTree.Report) childList.next());
        }
    }

    private void appendNavigation(Document document, Element parent, NavigationTree.Report item) {
        Element tag = document.createElement(item.isFolder() ? "container" : "item");

        tag.setAttribute(XML_ID, item.getId());
        tag.setAttribute(XML_LABEL, item.getLabel());
        tag.setAttribute(XML_DESCRIPTION, item.getDescription());
        parent.appendChild(tag);

        if (item.isFolder()) {
            NavigationTree.Folder container = (NavigationTree.Folder) item;

            appendAcl(document, tag, container.getAcl());

            Iterator childList = container.getChildren().iterator();
            while (childList.hasNext()) {
                appendNavigation(document, tag, (NavigationTree.Report) childList.next());
            }
        }
    }

    private void appendAcl(Document document, Element container, Acl acl) {
        String[] groups = acl.getGroups();

        // Don't bother to append the acl tag if there were no groups defined
        if (groups.length == 0)
            return;

        Element aclTag = document.createElement(XML_ACL);
        container.appendChild(aclTag);

        for (int x = 0; x < groups.length; x++) {
            Element groupTag = createTextElement(document, "group", groups[x]);
            aclTag.appendChild(groupTag);
        }
    }

    private void appendParamTags(Document document, Element parent, ParameterSupport source) {
        String[] paramNames = source.getParameterNames();

        Arrays.sort(paramNames);

        for (int x = 0; x < paramNames.length; x++) {
            Element param = document.createElement("param");
            Element paramName = createTextElement(document, "param-name", paramNames[x]);
            param.appendChild(paramName);

            String[] paramValues = source.getParameterValues(paramNames[x]);
            appendParamValues(document, param, paramValues);
            parent.appendChild(param);
        }
    }

    private void appendParamValues(Document document, Element param, String[] paramValues) {
        if (paramValues == null || paramValues.length == 0) {
            Element paramValue = createCDATAElement(document, "param-value", "");
            param.appendChild(paramValue);
        } else {
            for (int i = 0; i < paramValues.length; i++) {
                Element paramValue = createCDATAElement(document, "param-value", paramValues[i]);
                param.appendChild(paramValue);
            }
        }
    }

    private Element createTextElement(Document document, String tag, String textValue) {
        Element element = document.createElement(tag);
        Text text = document.createTextNode(textValue);
        element.appendChild(text);
        return element;
    }

    private Element createCDATAElement(Document document, String tag, String textValue) {
        Element element = document.createElement(tag);
        Text text = document.createCDATASection(HTMLUtils.encodeNonPrintingEntities(textValue));
        element.appendChild(text);
        return element;
    }

}
